@@ -35,9 +35,7 @@ test_that("@examples and @example interleave", {
     #' @example Rd-example-2.R
     NULL")[[1]]
 
-  verify_output(test_path("test-rd-examples-interleave.txt"), {
-    out$get_section("examples")
-  })
+  expect_snapshot_output(out$get_section("examples"))
 })
 
 test_that("@example does not introduce extra empty lines", {
@@ -51,29 +49,26 @@ test_that("@example does not introduce extra empty lines", {
 })
 
 test_that("@example gives warning if used instead of @examples", {
-  expect_warning(
+  expect_snapshot_warning(
     out <- roc_proc_text(rd_roclet(), "
       #' @name a
       #' @title a
       #' @example
       #' a <- 1
       #' a + b
-      NULL")[[1]],
-    "spans multiple lines"
+      NULL")[[1]]
   )
-
   expect_null(out$get_value("examples"))
 })
 
 test_that("warns if path doesn't exist", {
-  expect_warning(
+  expect_snapshot_warning(
     roc_proc_text(rd_roclet(), "
       #' @name a
       #' @title a
       #' @example this-path-doesnt-exist.R
       NULL
-    "),
-    "doesn't exist"
+    ")
   )
 })
 
@@ -87,9 +82,16 @@ test_that("@examplesIf", {
     #' and-this
     NULL")[[1]]
 
-  verify_output(test_path("test-rd-examplesIf.txt"), {
-    out$get_section("examples")
-  })
+  expect_snapshot_output(out$get_section("examples"))
+})
+
+test_that("@examplesIf warns about unparseable condition", {
+  expect_snapshot_warning(roc_proc_text(rd_roclet(), "
+    #' @name a
+    #' @title a
+    #' @examplesIf 1 +
+    #' maybe-run-this-code
+    NULL"))
 })
 
 test_that("% in @examples escaped before matching braces test (#213)", {
@@ -113,7 +115,7 @@ test_that("only % escaped in @examples", {
 
 test_that("multi-line macros in @example", {
   # https://github.com/r-lib/roxygen2/issues/974
-  out <- roxygen2:::roc_proc_text(roxygen2:::rd_roclet(), "
+  out <- roc_proc_text(rd_roclet(), "
     #' @name a
     #' @title a
     #'
