@@ -271,11 +271,31 @@ test_that("useful warnings", {
     "stop('Error')",
     "```"
   ))
+  path <- normalizePath(path)
+
   text <- sprintf("
     #' Title
     #' @includeRmd %s
     #' @name foobar
     NULL", path
   )
-  expect_snapshot_warning(roc_proc_text(rd_roclet(), text))
+  expect_snapshot(
+    . <- roc_proc_text(rd_roclet(), text),
+    transform = function(x) gsub(path, "<temp-path.Rmd>", x, fixed =TRUE)
+  )
+})
+
+test_that("sets width", {
+  skip_if_not(rmarkdown::pandoc_available("2.17"))
+
+  local_options(width = 123)
+  temp_rd <- withr::local_tempfile(lines = "`r getOption('width')`")
+
+  rox <- sprintf("
+    #' Title
+    #' @includeRmd %s
+    #' @name foobar
+    NULL", temp_rd)
+  out <- roc_proc_text(rd_roclet(), rox)[[1]]
+  expect_equal(out$get_value("details"), "80")
 })
